@@ -1,21 +1,19 @@
-// Copyright (c) 2012-2013, IGN France.
-// Copyright (c) 2012-2022, Oslandia.
-// SPDX-License-Identifier: LGPL-2.0-or-later
+
 
 // TODO : return of errors ! => error handler
 
-/**
- *
- * Minimal C API for SFCGAL
- *
- */
-
 /*--------------------------------------------------------------------------------------*
  *
  * Support for SFCGAL::Geometry class hierarchy
  *
  *--------------------------------------------------------------------------------------*/
 
+/**
+ * SRID type
+ * @ingroup capi
+ */
+typedef uint32_t srid_t;
+
 /**
  * sfcgal_geometry_t is an opaque pointer type that is used to represent a
  * pointer to SFCGAL::Geometry
@@ -67,6 +65,21 @@ sfcgal_set_geometry_validation(int enabled);
 sfcgal_geometry_type_t
 sfcgal_geometry_type_id(const sfcgal_geometry_t *);
 
+/**
+ * Returns the type of a given geometry as a string
+ * @post type is returned allocated and must be freed by the caller
+ * @ingroup capi
+ */
+void
+sfcgal_geometry_type(const sfcgal_geometry_t *, char **type, size_t *typeLen);
+
+/**
+ * Returns the dimension of a given geometry ( 0 : punctual, 1 : curve, ...)
+ * @ingroup capi
+ */
+int
+sfcgal_geometry_dimension(const sfcgal_geometry_t *);
+
 /**
  * Tests if the given geometry is valid or not
  * @ingroup capi
@@ -90,6 +103,27 @@ sfcgal_geometry_is_valid_detail(const sfcgal_geometry_t *geom,
                                 char                   **invalidity_reason,
                                 sfcgal_geometry_t      **invalidity_location);
 
+
+/**
+ * Tests if the given geometry is simple or not
+ * @ingroup capi
+ */
+int
+sfcgal_geometry_is_simple(const sfcgal_geometry_t *);
+
+/**
+ * Tests if the given geometry is simple or not
+ * And return details in case of complexity
+ * @param geom the input geometry
+ * @param complexity_reason input/output parameter. If non null, a
+ * null-terminated string could be allocated and contain reason of the
+ * complexity
+ * @ingroup capi
+ */
+int
+sfcgal_geometry_is_simple_detail(const sfcgal_geometry_t *geom,
+                                 char                   **complexity_reason);
+
 /**
  * Tests if the given geometry is 3D or not
  * @ingroup capi
@@ -111,6 +145,53 @@ sfcgal_geometry_is_measured(const sfcgal_geometry_t *);
 int
 sfcgal_geometry_is_empty(const sfcgal_geometry_t *);
 
+/**
+ * Drops the z coordinate of the geometry
+ * @post returns 1 if a Z value was present and has been removed. 0 otherwise.
+ * @ingroup capi
+ */
+int
+sfcgal_geometry_drop_z(sfcgal_geometry_t *);
+
+/**
+ * Drops the m coordinate of the geometry
+ * @post returns 1 if a M value was present and has been removed. 0 otherwise.
+ * @ingroup capi
+ */
+int
+sfcgal_geometry_drop_m(sfcgal_geometry_t *);
+
+/**
+ * Adds a z-dimension to the geometry, initialized to a preset value.
+ * Existing Z values remains unchanged.
+ * This has no effect on empty geometries.
+ * @param geom the input geometry
+ * @param defaultZ z-value to use
+ * @post returns 1 if a Z value was added. 0 otherwise.
+ * @ingroup capi
+ */
+int
+sfcgal_geometry_force_z(sfcgal_geometry_t *geom, double defaultZ);
+
+/**
+ * Adds a m-dimension to the geometry, initialized to a preset value.
+ * Existing M values remains unchanged.
+ * This has no effect on empty geometries.
+ * @param geom the input geometry
+ * @param defaultM m-value to use
+ * @post returns 1 if a M value was added. 0 otherwise.
+ * @ingroup capi
+ */
+int
+sfcgal_geometry_force_m(sfcgal_geometry_t *geom, double defaultM);
+
+/**
+ * Swaps the x and y coordinates of the geometry
+ * @ingroup capi
+ */
+void
+sfcgal_geometry_swap_xy(sfcgal_geometry_t *);
+
 /**
  * Returns a deep clone of the given geometry
  * @post returns a pointer to an allocated geometry that must be deallocated by
@@ -129,6 +210,16 @@ sfcgal_geometry_clone(const sfcgal_geometry_t *);
 void
 sfcgal_geometry_delete(sfcgal_geometry_t *);
 
+/**
+ * Returns the number of geometries of the given geometry collection
+ * @pre geometry must be a GeometryCollection.
+ * Otherwise, 1 is returned.  For empty geometries 0 is
+ * returned.
+ * @ingroup capi
+ */
+size_t
+sfcgal_geometry_num_geometries(const sfcgal_geometry_t *geometryCollection);
+
 /**
  * Returns a WKT representation of the given geometry using CGAL exact integer
  * fractions as coordinate values
@@ -167,12 +258,33 @@ void
 sfcgal_geometry_as_hexwkb(const sfcgal_geometry_t *, char **buffer,
                           size_t *len);
 
+/**
+ * Creates a VTK string of the given geometry
+ * @ingroup capi
+ */
+void
+sfcgal_geometry_as_vtk(const sfcgal_geometry_t *, char **buffer, size_t *len);
+
 /**
  * Creates a VTK file of the given geometry
  * @ingroup capi
  */
 void
-sfcgal_geometry_as_vtk(const sfcgal_geometry_t *, const char *filename);
+sfcgal_geometry_as_vtk_file(const sfcgal_geometry_t *, const char *filename);
+
+/**
+ * Creates a OBJ file of the given geometry
+ * @ingroup capi
+ */
+void
+sfcgal_geometry_as_obj_file(const sfcgal_geometry_t *, const char *filename);
+
+/**
+ * Creates a OBJ string of the given geometry
+ * @ingroup capi
+ */
+void
+sfcgal_geometry_as_obj(const sfcgal_geometry_t *, char **buffer, size_t *len);
 
 /**
  * Creates an empty point
@@ -264,6 +376,7 @@ sfcgal_linestring_num_points(const sfcgal_geometry_t *linestring);
 
 /**
  * Returns the ith point of a given LineString
+ * @param linestring the input LineString
  * @param i is the point index in the LineString
  * @pre linestring must be a LineString
  * @pre i >= and i < sfcgal_linestring_num_points
@@ -414,13 +527,6 @@ sfcgal_polygon_add_interior_ring(sfcgal_geometry_t *polygon,
 sfcgal_geometry_t *
 sfcgal_geometry_collection_create();
 
-/**
- * Returns the number of geometries of a given GeometryCollection
- * @pre collection is a GeometryCollection
- * @ingroup capi
- */
-size_t
-sfcgal_geometry_collection_num_geometries(const sfcgal_geometry_t *collection);
 
 /**
  * Returns the ith geometry of a GeometryCollection
@@ -434,6 +540,19 @@ const sfcgal_geometry_t *
 sfcgal_geometry_collection_geometry_n(const sfcgal_geometry_t *collection,
                                       size_t                   i);
 
+/**
+ * Set the ith geometry of a given GeometryCollection
+ * @pre collection is a GeometryCollection
+ * @pre i >= 0 and i < sfcgal_geometry_num_geometries( collection )
+ * The ownership of the geometry is taken. The caller is not responsible anymore
+ * of its deallocation.
+ * @ingroup capi
+ */
+void
+sfcgal_geometry_collection_set_geometry_n(sfcgal_geometry_t *collection,
+                                          sfcgal_geometry_t *geometry,
+                                          size_t             i);
+
 /**
  * Adds a Geometry to a given GeometryCollection
  * @pre collection must be a GeometryCollection
@@ -466,6 +585,13 @@ sfcgal_multi_linestring_create();
 sfcgal_geometry_t *
 sfcgal_multi_polygon_create();
 
+/**
+ * Creates an empty MultiSolid
+ * @ingroup capi
+ */
+sfcgal_geometry_t *
+sfcgal_multi_solid_create();
+
 /**
  * Creates an empty PolyhedralSurface
  * @ingroup capi
@@ -474,36 +600,52 @@ sfcgal_geometry_t *
 sfcgal_polyhedral_surface_create();
 
 /**
- * Returns the number of polygons of a given PolyhedralSurface
+ * Returns the number of patches of a given PolyhedralSurface
  * @pre polyhedral must be a PolyhedralSurface
  * @ingroup capi
  */
 size_t
-sfcgal_polyhedral_surface_num_polygons(const sfcgal_geometry_t *polyhedral);
+sfcgal_polyhedral_surface_num_patches(const sfcgal_geometry_t *polyhedral);
+
 
 /**
- * Returns the ith polygon of a given PolyhedralSurface
+ * Returns the ith patch of a given PolyhedralSurface
  * @pre polyhedral must be a PolyhedralSurface
- * @pre i >= 0 and i < sfcgal_polyhedral_surface_num_polygons(polyhedral)
+ * @pre i >= 0 and i < sfcgal_polyhedral_surface_num_patches(polyhedral)
  * @post the returned Polygon is not writable and must not be deallocated by the
  * caller
  * @ingroup capi
  */
 const sfcgal_geometry_t *
-sfcgal_polyhedral_surface_polygon_n(const sfcgal_geometry_t *polyhedral,
-                                    size_t                   i);
+sfcgal_polyhedral_surface_patch_n(const sfcgal_geometry_t *polyhedral,
+                                  size_t                   i);
+
 
 /**
- * Adds a Polygon to a given PolyhedralSurface
+ * Adds a Patch to a given PolyhedralSurface
  * @pre polyhedral must be a PolyhedralSurface
- * @pre polygon must be a Polygon
+ * @pre patch must be a Polygon
  * @post the ownership of the Polygon is taken. The caller is not responsible
  * anymore of its deallocation
  * @ingroup capi
  */
 void
-sfcgal_polyhedral_surface_add_polygon(sfcgal_geometry_t *polyhedral,
-                                      sfcgal_geometry_t *polygon);
+sfcgal_polyhedral_surface_add_patch(sfcgal_geometry_t *polyhedral,
+                                    sfcgal_geometry_t *patch);
+
+/**
+ * Set the ith patch of a given PolyhedralSurface
+ * @pre polyhedral must be a PolyhedralSurface.
+ * @pre patch must be a Polygon.
+ * @pre i >= 0 and i < sfcgal_polyhedral_surface_num_patches(polyhedral)
+ * The ownership of the polygon is taken. The caller is not responsible anymore
+ * of its deallocation.
+ * @ingroup capi
+ */
+void
+sfcgal_polyhedral_surface_set_patch_n(sfcgal_geometry_t *polyhedral,
+                                      sfcgal_geometry_t *patch, size_t i);
+
 
 /**
  * Creates an empty TriangulatedSurface
@@ -513,35 +655,51 @@ sfcgal_geometry_t *
 sfcgal_triangulated_surface_create();
 
 /**
- * Returns the number of triangles of a given TriangulatedSurface
+ * Returns the number of patches of a given TriangulatedSurface
  * @pre tin must be a TriangulatedSurface
  * @ingroup capi
  */
 size_t
-sfcgal_triangulated_surface_num_triangles(const sfcgal_geometry_t *tin);
+sfcgal_triangulated_surface_num_patches(const sfcgal_geometry_t *tin);
+
 
 /**
- * Returns the ith Triangle of a given TriangulatedSurface
+ * Returns the ith Patch of a given TriangulatedSurface
  * @pre tin must be a TriangulatedSurface
- * @pre i >= 0 and i < sfcgal_triangulated_surface_num_triangles( tin )
+ * @pre i >= 0 and i < sfcgal_triangulated_surface_num_patches( tin )
  * @post the returned Triangle is not writable and must not be deallocated by
  * the caller
  * @ingroup capi
  */
 const sfcgal_geometry_t *
-sfcgal_triangulated_surface_triangle_n(const sfcgal_geometry_t *tin, size_t i);
+sfcgal_triangulated_surface_patch_n(const sfcgal_geometry_t *tin, size_t i);
+
 
 /**
- * Adds a Triangle to a given TriangulatedSurface
+ * Set the ith patch of a given TriangulatedSurface
  * @pre tin must be a TriangulatedSurface
- * @pre triangle must be a Triangle
+ * @pre patch must be a Triangle.
+ * @pre i >= 0 and i < sfcgal_triangulated_surface_num_patches( tin )
+ * The ownership of the triangle is taken. The caller is not responsible anymore
+ * of its deallocation.
+ * @ingroup capi
+ */
+void
+sfcgal_triangulated_surface_set_patch_n(sfcgal_geometry_t *tin,
+                                        sfcgal_geometry_t *patch, size_t i);
+
+/**
+ * Adds a Patch to a given TriangulatedSurface
+ * @pre tin must be a TriangulatedSurface
+ * @pre patch must be a Triangle
  * @post the ownership of the Triangle is taken. The caller is not responsible
  * anymore of its deallocation
  * @ingroup capi
  */
 void
-sfcgal_triangulated_surface_add_triangle(sfcgal_geometry_t *tin,
-                                         sfcgal_geometry_t *triangle);
+sfcgal_triangulated_surface_add_patch(sfcgal_geometry_t *tin,
+                                      sfcgal_geometry_t *patch);
+
 
 /**
  * Creates an empty Solid
@@ -591,6 +749,18 @@ void
 sfcgal_solid_add_interior_shell(sfcgal_geometry_t *solid,
                                 sfcgal_geometry_t *shell);
 
+/**
+ * Set the exterior shell of a given Solid
+ * @pre solid must be a Solid
+ * @pre shell must be a PolyhedralSurface
+ * @post the ownership of the shell is taken. The caller is not responsible
+ * anymore of its deallocation
+ * @ingroup capi
+ */
+void
+sfcgal_solid_set_exterior_shell(sfcgal_geometry_t *solid,
+                                sfcgal_geometry_t *shell);
+
 /**
  * Gets the validity flag of the geometry.
  */
@@ -616,8 +786,6 @@ sfcgal_geometry_force_valid(sfcgal_geometry_t *geom, int valid);
  */
 typedef void sfcgal_prepared_geometry_t;
 
-typedef uint32_t srid_t;
-
 /**
  * Creates an empty PreparedGeometry
  * @ingroup capi
@@ -682,8 +850,11 @@ sfcgal_prepared_geometry_set_srid(sfcgal_prepared_geometry_t *prepared, srid_t);
 
 /**
  * Returns an EWKT representation of the given PreparedGeometry
- * @param num_decimals number of decimals. -2 for a variable number of decimals.
- * -1 for an exact representation
+ * @param[in] prepared the input geometry
+ * @param[in] num_decimals number of decimals. -2 for a variable number of
+ * decimals.
+ * @param[out] buffer the EWKT buffer
+ * @param[out] len the size of the buffer
  * @post buffer is returned allocated and must be freed by the caller
  * @ingroup capi
  */
@@ -907,7 +1078,7 @@ sfcgal_geometry_extrude(const sfcgal_geometry_t *geom, double ex, double ey,
  * Convert a PolyhedralSurface to a Solid
  * @pre isValid(geom) == true
  * @post isValid(return) == true
- * @ingroup detail
+ * @ingroup capi
  */
 sfcgal_geometry_t *
 sfcgal_geometry_make_solid(const sfcgal_geometry_t *geom);
@@ -1033,7 +1204,18 @@ sfcgal_geometry_t *
 sfcgal_geometry_approximate_medial_axis(const sfcgal_geometry_t *geom);
 
 /**
- * Tests the coverage of geom1 and geom2
+ * Returns the straight skeleton partition for the given Polygon
+ * @pre isValid(geom) == true
+ * @pre geom must be a Polygon, Triangle or MultiPolygon
+ * @post isValid(return) == true
+ * @ingroup capi
+ */
+sfcgal_geometry_t *
+sfcgal_geometry_straight_skeleton_partition(const sfcgal_geometry_t *geom,
+                                            bool autoOrientation);
+
+/**
+ * Tests the 2D coverage of geom1 and geom2
  * @pre isValid(geom1) == true
  * @pre isValid(geom2) == true
  * @ingroup capi
@@ -1065,6 +1247,122 @@ sfcgal_geometry_t *
 sfcgal_geometry_line_sub_string(const sfcgal_geometry_t *geom, double start,
                                 double end);
 
+
+
+/**
+ * Returns the 3D alpha wrapping of a geometry
+ * @pre isValid(geom) == true
+ * @pre relative_alpha >= 0
+ * @pre relative_offset >= 0
+ * @post isValid(return) == true
+ * @param geom input geometry
+ * @param relative_alpha This parameter is used to determine which features will
+ *  appear in the output. A small relative_alpha will produce an output less
+ *  complex but less faithful to the input.
+ * @param relative_offset  This parameter controls the tightness of the result.
+ *  A large relative_offset parameter will tend to better preserve sharp
+ * features as projection. If this parameter is equal to 0, it is computed from
+ * the alpha parameter
+ * @return A PolyhedralSurface representing the 3D alpha wrapping of the
+ * geometry
+ * @ingroup capi
+ */
+sfcgal_geometry_t *
+sfcgal_geometry_alpha_wrapping_3d(const sfcgal_geometry_t *geom,
+                                  size_t                   relative_alpha,
+                                  size_t                   relative_offset);
+
+/**
+ * Returns the envelope of geom
+ * @pre isValid(geom) == true
+ * @post isValid(return) == true
+ * @ingroup capi
+ */
+sfcgal_geometry_t *
+sfcgal_geometry_envelope(const sfcgal_geometry_t *geom);
+
+/**
+ * Returns the 3d envelope of geom
+ * @pre isValid(geom) == true
+ * @post isValid(return) == true
+ * @ingroup capi
+ */
+sfcgal_geometry_t *
+sfcgal_geometry_envelope_3d(const sfcgal_geometry_t *geom);
+
+/**
+ * Returns the 2D length of geom
+ * @pre isValid(geom) == true
+ * @post isValid(return) == true
+ * @ingroup capi
+ */
+double
+sfcgal_geometry_length(const sfcgal_geometry_t *geom);
+
+/**
+ * Returns the 3D length of geom
+ * @pre isValid(geom) == true
+ * @post isValid(return) == true
+ * @ingroup capi
+ */
+double
+sfcgal_geometry_length_3d(const sfcgal_geometry_t *geom);
+
+/**
+ * Returns the boundary of geom
+ * @pre isValid(geom) == true
+ * @post the caller is responsible for the boundary deallocation.
+ * @ingroup capi
+ */
+sfcgal_geometry_t *
+sfcgal_geometry_boundary(const sfcgal_geometry_t *geom);
+
+/**
+ * Returns a Point representing the geometry centroid
+ * @pre isValid(geom) == true
+ * @post isValid(return) == true
+ * @ingroup capi
+ */
+sfcgal_geometry_t *
+sfcgal_geometry_centroid(const sfcgal_geometry_t *geom);
+
+/**
+ * Returns a Point representing the geometry centroid
+ * @pre isValid(geom) == true
+ * @post isValid(return) == true
+ * @ingroup capi
+ */
+sfcgal_geometry_t *
+sfcgal_geometry_centroid_3d(const sfcgal_geometry_t *geom);
+
+/**
+ * Returns true if geom1 is equals to geom2.
+ *
+ * For each point of geom1 there is a point in geom2.
+ * @pre isValid(geom) == true
+ * @post isValid(return) == true
+ * @ingroup capi
+ */
+int
+sfcgal_geometry_is_equals(const sfcgal_geometry_t *geom1,
+                          const sfcgal_geometry_t *geom2);
+
+/**
+ * Returns true if geom1 is almost equals to geom2.
+ *
+ * For each point of geom1 there is a point in geom2 within tolerance distance.
+ * @param geom1 the first geometry
+ * @param geom2 the second geometry
+ * @param tolerance the tolerance
+ * @pre isValid(geom) == true
+ * @post isValid(return) == true
+ * @ingroup capi
+ */
+int
+sfcgal_geometry_is_almost_equals(const sfcgal_geometry_t *geom1,
+                                 const sfcgal_geometry_t *geom2,
+                                 double                   tolerance);
+
 /**
  * Returns the y monotone partition of a geometry (polygon without hole)
  * @pre isValid(geom) == true
@@ -1120,7 +1418,7 @@ sfcgal_geometry_visibility_point(const sfcgal_geometry_t *polygon,
  * @param polygon input geometry
  * @param pointA input geometry
  * @param pointB input geometry
- * @ingroup public_api
+ * @ingroup capi
  * @pre polygon is a valid geometry
  * @pre pointA and pointB must be vertices of poly, adjacents and respect the
  * direction
@@ -1130,6 +1428,189 @@ sfcgal_geometry_visibility_segment(const sfcgal_geometry_t *polygon,
                                    const sfcgal_geometry_t *pointA,
                                    const sfcgal_geometry_t *pointB);
 
+/**
+ * Buffer3D types
+ * @ingroup capi
+ */
+typedef enum {
+  SFCGAL_BUFFER3D_ROUND,
+  SFCGAL_BUFFER3D_CYLSPHERE,
+  SFCGAL_BUFFER3D_FLAT
+} sfcgal_buffer3d_type_t;
+
+/**
+ * Computes a 3D buffer around a geometry
+ * @param geom The input geometry (must be a Point or LineString)
+ * @param radius The buffer radius
+ * @param segments The number of segments to use for approximating curved
+ * surfaces
+ * @param buffer_type The type of buffer to compute (ROUND, CYLSPHERE, or FLAT)
+ * @return A new geometry representing the 3D buffer
+ * @pre isValid(geom) == true
+ * @pre radius > 0
+ * @pre segments > 3
+ * @post isValid(return) == true
+ * @post The returned geometry must be deallocated by the caller
+ * @ingroup capi
+ */
+sfcgal_geometry_t *
+sfcgal_geometry_buffer3d(const sfcgal_geometry_t *geom, double radius,
+                         int segments, sfcgal_buffer3d_type_t buffer_type);
+
+/*--------------------------------------------------------------------------------------*
+ *
+ * Transformation
+ *
+ *--------------------------------------------------------------------------------------*/
+
+/**
+ * Rotates a geometry around the origin (0,0,0) by a given angle
+ * @param geom The geometry to rotate
+ * @param angle Rotation angle in radians
+ * @return The rotated geometry
+ */
+sfcgal_geometry_t *
+sfcgal_geometry_rotate(const sfcgal_geometry_t *geom, double angle);
+
+/**
+ * Rotates a geometry around a specified point by a given angle
+ * @param geom The geometry to rotate
+ * @param angle Rotation angle in radians
+ * @param cx X-coordinate of the center point
+ * @param cy Y-coordinate of the center point
+ * @return The rotated geometry
+ */
+sfcgal_geometry_t *
+sfcgal_geometry_rotate_2d(const sfcgal_geometry_t *geom, double angle,
+                          double cx, double cy);
+
+/**
+ * Rotates a 3D geometry around a specified axis by a given angle
+ * @param geom The geometry to rotate
+ * @param angle Rotation angle in radians
+ * @param ax X-coordinate of the axis vector
+ * @param ay Y-coordinate of the axis vector
+ * @param az Z-coordinate of the axis vector
+ * @return The rotated geometry
+ */
+sfcgal_geometry_t *
+sfcgal_geometry_rotate_3d(const sfcgal_geometry_t *geom, double angle,
+                          double ax, double ay, double az);
+
+/**
+ * Rotates a 3D geometry around a specified axis and center point by a given
+ * angle
+ * @param geom The geometry to rotate
+ * @param angle Rotation angle in radians
+ * @param ax X-coordinate of the axis vector
+ * @param ay Y-coordinate of the axis vector
+ * @param az Z-coordinate of the axis vector
+ * @param cx X-coordinate of the center point
+ * @param cy Y-coordinate of the center point
+ * @param cz Z-coordinate of the center point
+ * @return The rotated geometry
+ */
+sfcgal_geometry_t *
+sfcgal_geometry_rotate_3d_around_center(const sfcgal_geometry_t *geom,
+                                        double angle, double ax, double ay,
+                                        double az, double cx, double cy,
+                                        double cz);
+
+/**
+ * Rotates a geometry around the X axis by a given angle
+ * @param geom The geometry to rotate
+ * @param angle Rotation angle in radians
+ * @return The rotated geometry
+ */
+sfcgal_geometry_t *
+sfcgal_geometry_rotate_x(const sfcgal_geometry_t *geom, double angle);
+
+/**
+ * Rotates a geometry around the Y axis by a given angle
+ * @param geom The geometry to rotate
+ * @param angle Rotation angle in radians
+ * @return The rotated geometry
+ */
+sfcgal_geometry_t *
+sfcgal_geometry_rotate_y(const sfcgal_geometry_t *geom, double angle);
+
+/**
+ * Rotates a geometry around the Z axis by a given angle
+ * @param geom The geometry to rotate
+ * @param angle Rotation angle in radians
+ * @return The rotated geometry
+ */
+sfcgal_geometry_t *
+sfcgal_geometry_rotate_z(const sfcgal_geometry_t *geom, double angle);
+
+/**
+ * Scale a geometry by a given factor
+ * @param geom The geometry to scale
+ * @param s Scale factor
+ * @return The scaled geometry
+ */
+sfcgal_geometry_t *
+sfcgal_geometry_scale(const sfcgal_geometry_t *geom, double s);
+
+/**
+ * Scale a geometry by different factors for each dimension
+ * @param geom The geometry to scale
+ * @param sx Scale factor for x dimension
+ * @param sy Scale factor for y dimension
+ * @param sz Scale factor for z dimension
+ * @return The scaled geometry
+ */
+sfcgal_geometry_t *
+sfcgal_geometry_scale_3d(const sfcgal_geometry_t *geom, double sx, double sy,
+                         double sz);
+
+/**
+ * Scale a geometry by different factors for each dimension around a center
+ * point
+ * @param geom The geometry to scale
+ * @param sx Scale factor for x dimension
+ * @param sy Scale factor for y dimension
+ * @param sz Scale factor for z dimension
+ * @param cx X-coordinate of the center point
+ * @param cy Y-coordinate of the center point
+ * @param cz Z-coordinate of the center point
+ * @return The scaled geometry
+ */
+sfcgal_geometry_t *
+sfcgal_geometry_scale_3d_around_center(const sfcgal_geometry_t *geom, double sx,
+                                       double sy, double sz, double cx,
+                                       double cy, double cz);
+
+/**
+ * Translate a geometry by a 3D vector
+ * @param geom the geometry to translate
+ * @param dx x component of the translation vector
+ * @param dy y component of the translation vector
+ * @param dz z component of the translation vector
+ */
+sfcgal_geometry_t *
+sfcgal_geometry_translate_3d(const sfcgal_geometry_t *geom, double dx,
+                             double dy, double dz);
+
+/**
+ * Translate a geometry by a 2D vector
+ * @param geom the geometry to translate
+ * @param dx x component of the translation vector
+ * @param dy y component of the translation vector
+ */
+sfcgal_geometry_t *
+sfcgal_geometry_translate_2d(const sfcgal_geometry_t *geom, double dx,
+                             double dy);
+
+/**
+ * Simplify a geometry
+ * @param geom the geometry to simplify
+ * @param threshold threshold parameter
+ * @param preserveTopology preserve the topology
+ */
+sfcgal_geometry_t *
+sfcgal_geometry_simplify(const sfcgal_geometry_t *geom, double threshold,
+                         bool preserveTopology);
 /*--------------------------------------------------------------------------------------*
  *
  * Error handling
@@ -1166,7 +1647,8 @@ typedef void *(*sfcgal_alloc_handler_t)(size_t);
 typedef void (*sfcgal_free_handler_t)(void *);
 
 /**
- * Sets the error handlers. These callbacks are called on warning or error
+ * Sets the allocation handlers. These functions are called on memory allocation
+ * and deallocation.
  * @param malloc_handler is the function to call for memory allocation. The
  * default behaviour is to call malloc()
  * @param free_handler is the function to call for memory deallocation. The
@@ -1177,6 +1659,14 @@ void
 sfcgal_set_alloc_handlers(sfcgal_alloc_handler_t malloc_handler,
                           sfcgal_free_handler_t  free_handler);
 
+/**
+ * Delete a buffer previously allocated and returned by SFCGAL.
+ * @param buffer a buffer previously allocated and returned by SFCGAL
+ * @ingroup capi
+ */
+void
+sfcgal_free_buffer(void *buffer);
+
 /*--------------------------------------------------------------------------------------*
  *
  * Init
